@@ -26,6 +26,8 @@
 
 #include <glib-object.h>
 
+G_BEGIN_DECLS
+
 #define GOSSIP_TYPE_ACCOUNT         (gossip_account_get_gtype ())
 #define GOSSIP_ACCOUNT(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GOSSIP_TYPE_ACCOUNT, GossipAccount))
 #define GOSSIP_ACCOUNT_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), GOSSIP_TYPE_ACCOUNT, GossipAccountClass))
@@ -35,6 +37,7 @@
 
 typedef struct _GossipAccount      GossipAccount;
 typedef struct _GossipAccountClass GossipAccountClass;
+typedef struct _GossipAccountParam GossipAccountParam;
 
 struct _GossipAccount {
 	GObject parent;
@@ -51,11 +54,6 @@ typedef enum {
 	GOSSIP_ACCOUNT_PARAM_FLAG_ALL         = (1 << 3) - 1
 } GossipAccountParamFlags;
 
-typedef struct {
-	GossipAccountParamFlags flags;
-	GValue                  g_value;
-} GossipAccountParam;
-
 typedef enum {
 	GOSSIP_ACCOUNT_TYPE_JABBER,
 	GOSSIP_ACCOUNT_TYPE_AIM,
@@ -66,12 +64,20 @@ typedef enum {
 	GOSSIP_ACCOUNT_TYPE_COUNT
 } GossipAccountType;
 
+struct _GossipAccountParam {
+	GossipAccountParamFlags flags;
+	GValue                  g_value;
+	gboolean                modified;
+};
+
 typedef void (*GossipAccountParamFunc) (GossipAccount      *account,
-					gchar              *param_name,
+					const gchar        *param_name,
 					GossipAccountParam *param,
 					gpointer            user_data);
 
 GType             gossip_account_get_gtype         (void) G_GNUC_CONST;
+
+/* Account parameter functions */
 void              gossip_account_param_new         (GossipAccount           *account,
 						    const gchar             *first_param_name,
 						    ...);
@@ -92,26 +98,39 @@ const GValue *    gossip_account_param_get_g_value (GossipAccount           *acc
 						    const gchar             *param_name);
 gboolean          gossip_account_has_param         (GossipAccount           *account,
 						    const gchar             *param_name);
+GList *           gossip_account_param_get_all     (GossipAccount           *account);
+GossipAccountParam *
+                  gossip_account_param_get_param   (GossipAccount           *account,
+						    const gchar             *param_name);
 void              gossip_account_param_foreach     (GossipAccount           *account,
 						    GossipAccountParamFunc   callback,
 						    gpointer                 user_data);
+
+/* Account functions */
 GossipAccountType gossip_account_get_type          (GossipAccount           *account);
-const gchar *     gossip_account_get_id            (GossipAccount           *account);
 const gchar *     gossip_account_get_name          (GossipAccount           *account);
 gboolean          gossip_account_get_auto_connect  (GossipAccount           *account);
 gboolean          gossip_account_get_use_proxy     (GossipAccount           *account);
-void              gossip_account_set_id            (GossipAccount           *account,
-						    const gchar             *id);
 void              gossip_account_set_name          (GossipAccount           *account,
 						    const gchar             *name);
 void              gossip_account_set_auto_connect  (GossipAccount           *account,
 						    gboolean                 auto_connect);
 void              gossip_account_set_use_proxy     (GossipAccount           *account,
 						    gboolean                 use_proxy);
+#ifdef HAVE_TELEPATHY
+const gchar *     gossip_account_get_protocol      (GossipAccount           *account);
+const gchar *     gossip_account_get_cmgr_name     (GossipAccount           *account);
+void              gossip_account_set_protocol      (GossipAccount           *account,
+						    const gchar             *protocol);
+void              gossip_account_set_cmgr_name     (GossipAccount           *account,
+						    const gchar             *cmgr_name);
+#endif
 guint             gossip_account_hash              (gconstpointer            key);
-gboolean          gossip_account_equal             (gconstpointer            v1,
-						    gconstpointer            v2);
+gboolean          gossip_account_equal             (gconstpointer            a,
+						    gconstpointer            b);
 const gchar *     gossip_account_type_to_string    (GossipAccountType        type);
 GossipAccountType gossip_account_string_to_type    (const gchar             *str);
 
+G_END_DECLS
+
 #endif /* __GOSSIP_ACCOUNT_H__ */
