@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005-2006 Imendio AB
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,15 +16,26 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <glade/glade.h>
 
-#include <libgossip/gossip-session.h>
 #include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-event.h>
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-session.h>
+#include <libgossip/gossip-message.h>
+#include <libgossip/gossip-event-manager.h>
+#include <libgossip/gossip-utils.h>
 
 #include "gossip-app.h"
 #include "gossip-cell-renderer-expander.h"
@@ -35,14 +46,11 @@
 #include "gossip-contact-list.h"
 #include "gossip-edit-contact-dialog.h"
 #include "gossip-ft-window.h"
-#include "gossip-log.h"
 #include "gossip-log-window.h"
 #include "gossip-marshal.h"
 #include "gossip-sound.h"
 #include "gossip-stock.h"
-
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
-                       GOSSIP_TYPE_CONTACT_LIST, GossipContactListPriv))
+#include "gossip-ui-utils.h"
 
 #define DEBUG_DOMAIN "ContactList"
 
@@ -59,6 +67,8 @@
 /* Time after connecting which we wait before active users are enabled */
 #define ACTIVE_USER_WAIT_TO_ENABLE_TIME 5000
 
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CONTACT_LIST, GossipContactListPriv))
+
 struct _GossipContactListPriv {
 	GossipSession       *session;
 
@@ -130,11 +140,11 @@ static void     contact_list_contact_update                  (GossipContactList
 static void     contact_list_contact_added_cb                (GossipSession          *session,
 							      GossipContact          *contact,
 							      GossipContactList      *list);
-static void     contact_list_contact_updated_cb              (GossipSession          *session,
-							      GossipContact          *contact,
+static void     contact_list_contact_updated_cb              (GossipContact          *contact,
+							      GParamSpec             *param,
 							      GossipContactList      *list);
-static void     contact_list_contact_presence_updated_cb     (GossipSession          *session,
-							      GossipContact          *contact,
+static void     contact_list_contact_groups_updated_cb       (GossipContact          *contact,
+							      GParamSpec             *param,
 							      GossipContactList      *list);
 static void     contact_list_contact_removed_cb              (GossipSession          *session,
 							      GossipContact          *contact,
@@ -257,6 +267,9 @@ static void     contact_list_action_activated                (GossipContactList
 							      GossipContact          *contact);
 static void     contact_list_action_entry_activate_cb        (GtkWidget              *entry,
 							      GtkDialog              *dialog);
+static void     contact_list_action_remove_response_cb       (GtkWidget              *dialog,
+							      gint                    response,
+							      GossipContactList      *list);
 static void     contact_list_action_remove_selected          (GossipContactList      *list);
 static void     contact_list_action_invite_selected          (GossipContactList      *list);
 static void     contact_list_action_rename_group_selected    (GossipContactList      *list);
@@ -500,14 +513,6 @@ gossip_contact_list_init (GossipContactList *list)
 			  "contact-added",
 			  G_CALLBACK (contact_list_contact_added_cb),
 			  list);
-	g_signal_connect (priv->session,
-			  "contact-updated",
-			  G_CALLBACK (contact_list_contact_updated_cb),
-			  list);
-	g_signal_connect (priv->session,
-			  "contact-presence-updated",
-			  G_CALLBACK (contact_list_contact_presence_updated_cb),
-			  list);
 	g_signal_connect (priv->session,
 			  "contact-removed",
 			  G_CALLBACK (contact_list_contact_removed_cb),
@@ -663,6 +668,7 @@ contact_list_contact_update (GossipContactList *list,
 	const GdkPixbuf       *pixbuf;
 	GdkPixbuf             *pixbuf_composing;
 	GdkPixbuf             *pixbuf_presence;
+	GdkPixbuf             *pixbuf_avatar;
 
 	priv = GET_PRIV (list);
 
@@ -790,6 +796,7 @@ contact_list_contact_update (GossipContactList *list,
 	pixbuf_presence = gossip_pixbuf_for_contact (contact);
 	pixbuf_composing = gossip_pixbuf_from_stock (GOSSIP_STOCK_TYPING,
 						     GTK_ICON_SIZE_MENU);
+	pixbuf_avatar = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
 
 	pixbuf = pixbuf_presence;
 
@@ -806,11 +813,16 @@ contact_list_contact_update (GossipContactList *list,
 				    COL_PIXBUF_STATUS, pixbuf,
 				    COL_STATUS, gossip_contact_get_status (contact),
 				    COL_IS_ONLINE, now_online,
+				    COL_NAME, gossip_contact_get_name (contact),
+				    COL_PIXBUF_AVATAR, pixbuf_avatar,
 				    -1);
 	}
 
 	g_object_unref (pixbuf_presence);
 	g_object_unref (pixbuf_composing);
+	if (pixbuf_avatar) {
+		g_object_unref (pixbuf_avatar);
+	}
 
 	if (priv->show_active && do_set_active) {
 		contact_list_contact_set_active (list, contact, do_set_active, do_set_refresh);
@@ -839,7 +851,6 @@ contact_list_contact_added_cb (GossipSession     *session,
 {
 	GossipContactListPriv *priv;
 	GtkTreeModel          *model;
-	GList                 *groups;
 	gboolean               has_events;
 
 	priv = GET_PRIV (list);
@@ -847,6 +858,23 @@ contact_list_contact_added_cb (GossipSession     *session,
 	gossip_debug (DEBUG_DOMAIN, "Contact:'%s' added",
 		      gossip_contact_get_name (contact));
 
+	/* Connect notifications for contact updates */
+	g_signal_connect (contact, "notify::groups",
+			  G_CALLBACK (contact_list_contact_groups_updated_cb),
+			  list);
+	g_signal_connect (contact, "notify::presences",
+			  G_CALLBACK (contact_list_contact_updated_cb),
+			  list);
+	g_signal_connect (contact, "notify::name",
+			  G_CALLBACK (contact_list_contact_updated_cb),
+			  list);
+	g_signal_connect (contact, "notify::avatar",
+			  G_CALLBACK (contact_list_contact_updated_cb),
+			  list);
+	g_signal_connect (contact, "notify::type",
+			  G_CALLBACK (contact_list_contact_updated_cb),
+			  list);
+
 	has_events = g_hash_table_lookup (priv->flash_table, contact) != NULL;
 
 	if (!has_events &&
@@ -857,20 +885,16 @@ contact_list_contact_added_cb (GossipSession     *session,
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
-	groups = gossip_contact_get_groups (contact);
-
 	contact_list_add_contact (list, contact);
 }
 
 static void
-contact_list_contact_updated_cb (GossipSession     *session,
-				 GossipContact     *contact,
-				 GossipContactList *list)
+contact_list_contact_groups_updated_cb (GossipContact     *contact,
+					GParamSpec        *param,
+					GossipContactList *list)
 {
 	GossipContactListPriv *priv;
 	GossipContactType      type;
-	GossipPresence        *presence;
-	GtkTreeModel          *model;
 
 	priv = GET_PRIV (list);
 
@@ -883,16 +907,9 @@ contact_list_contact_updated_cb (GossipSession     *session,
 		return;
 	}
 
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
-	presence = gossip_contact_get_active_presence (contact);
-
-	gossip_debug (DEBUG_DOMAIN, "Contact:'%s' updated",
+	gossip_debug (DEBUG_DOMAIN, "Contact:'%s' groups updated",
 		      gossip_contact_get_name (contact));
 
-	if (!priv->show_offline && !gossip_contact_is_online (contact)) {
-		return;
-	}
-
 	/* We do this to make sure the groups are correct, if not, we
 	 * would have to check the groups already set up for each
 	 * contact and then see what has been updated.
@@ -902,13 +919,10 @@ contact_list_contact_updated_cb (GossipSession     *session,
 }
 
 static void
-contact_list_contact_presence_updated_cb (GossipSession     *session,
-					  GossipContact     *contact,
-					  GossipContactList *list)
+contact_list_contact_updated_cb (GossipContact     *contact,
+				 GParamSpec        *param,
+				 GossipContactList *list)
 {
-	gossip_debug (DEBUG_DOMAIN, "Contact:'%s' presence update",
-		      gossip_contact_get_name (contact));
-
 	contact_list_contact_update (list, contact);
 }
 
@@ -1025,7 +1039,12 @@ contact_list_contact_active_new (GossipContactList *list,
 	g_return_val_if_fail (list != NULL, NULL);
 	g_return_val_if_fail (contact != NULL, NULL);
 
-	data = g_new0 (ShowActiveData, 1);
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Contact:'%s' now active, and %s be removed",
+		      gossip_contact_get_name (contact), 
+		      remove ? "WILL" : "WILL NOT");
+	
+	data = g_slice_new0 (ShowActiveData);
 
 	data->list = g_object_ref (list);
 	data->contact = g_object_ref (contact);
@@ -1043,7 +1062,7 @@ contact_list_contact_active_free (ShowActiveData *data)
 	g_object_unref (data->contact);
 	g_object_unref (data->list);
 
-	g_free (data);
+	g_slice_free (ShowActiveData, data);
 }
 
 static gboolean
@@ -1058,13 +1077,17 @@ contact_list_contact_active_cb (ShowActiveData *data)
 	if (data->remove &&
 	    !priv->show_offline &&
 	    !gossip_contact_is_online (data->contact)) {
-		gossip_debug (DEBUG_DOMAIN, "Remove item (active timeout)!");
+		gossip_debug (DEBUG_DOMAIN, 
+			      "Contact:'%s' active timeout, removing item",
+			      gossip_contact_get_name (data->contact));
 		contact_list_remove_contact (data->list,
 					     data->contact,
 					     FALSE);
 	}
 
-	gossip_debug (DEBUG_DOMAIN, "Setting contact to no longer be active");
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Contact:'%s' no longer active",
+		      gossip_contact_get_name (data->contact));
 	contact_list_contact_set_active (data->list,
 					 data->contact,
 					 FALSE,
@@ -2397,14 +2420,31 @@ contact_list_action_entry_activate_cb (GtkWidget *entry,
 	gtk_dialog_response (dialog, GTK_RESPONSE_OK);
 }
 
+static void
+contact_list_action_remove_response_cb (GtkWidget         *dialog,
+					gint               response,
+					GossipContactList *list)
+{
+	if (response == GTK_RESPONSE_YES) {
+		GossipContact         *contact;
+		GossipContactListPriv *priv;
+
+		priv = GET_PRIV (list);
+
+		contact = g_object_get_data (G_OBJECT (dialog), "contact");
+		gossip_session_remove_contact (priv->session, contact);
+	}
+
+	gtk_widget_destroy (dialog);
+}
+
 static void
 contact_list_action_remove_selected (GossipContactList *list)
 {
 	GossipContactListPriv *priv;
 	GossipContact         *contact;
-	gchar                 *name;
 	GtkWidget             *dialog;
-	gint                   response;
+	gchar                 *name;
 
 	priv = GET_PRIV (list);
 
@@ -2416,7 +2456,7 @@ contact_list_action_remove_selected (GossipContactList *list)
 	name = g_strdup_printf ("<b>%s</b>\n",
 				gossip_contact_get_name (contact));
 
-	dialog = gtk_message_dialog_new (NULL,
+	dialog = gtk_message_dialog_new (GTK_WINDOW (gossip_app_get_window ()),
 					 0,
 					 GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_NONE,
@@ -2429,10 +2469,8 @@ contact_list_action_remove_selected (GossipContactList *list)
 	g_free (name);
 
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-				GTK_STOCK_CANCEL,
-				GTK_RESPONSE_NO,
-				GTK_STOCK_REMOVE,
-				GTK_RESPONSE_YES,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
+				GTK_STOCK_REMOVE, GTK_RESPONSE_YES,
 				NULL);
 
 	g_object_set (GTK_MESSAGE_DIALOG (dialog)->label,
@@ -2440,17 +2478,13 @@ contact_list_action_remove_selected (GossipContactList *list)
 		      "wrap", FALSE,
 		      NULL);
 
-	gtk_window_set_transient_for (GTK_WINDOW (dialog),
-				      GTK_WINDOW (gossip_app_get_window ()));
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	g_object_set_data_full (G_OBJECT (dialog), "contact", contact, g_object_unref);
 
-	if (response == GTK_RESPONSE_YES) {
-		gossip_session_remove_contact (priv->session,
-					       contact);
-	}
-
-	g_object_unref (contact);
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (contact_list_action_remove_response_cb),
+			  list);
+	
+	gtk_widget_show_all (dialog);
 }
 
 static void
@@ -2466,6 +2500,31 @@ contact_list_action_invite_selected (GossipContactList *list)
 	gossip_chat_invite_dialog_show (contact, 0);
 }
 
+static void
+contact_list_action_rename_group_response_cb (GtkWidget         *dialog,
+					      gint               response,
+					      GossipContactList *list)
+{
+	if (response == GTK_RESPONSE_OK) {
+		GtkWidget   *entry;
+		const gchar *name;
+		const gchar *group;
+		
+		entry = g_object_get_data (G_OBJECT (dialog), "entry");
+		group = g_object_get_data (G_OBJECT (dialog), "group");
+		name = gtk_entry_get_text (GTK_ENTRY (entry));
+		
+		if (!G_STR_EMPTY (name)) {
+			GossipContactListPriv *priv;
+
+			priv = GET_PRIV (list);
+			gossip_session_rename_group (priv->session, group, name);
+		}
+	}
+
+	gtk_widget_destroy (dialog);
+}
+
 static void
 contact_list_action_rename_group_selected (GossipContactList *list)
 {
@@ -2483,55 +2542,42 @@ contact_list_action_rename_group_selected (GossipContactList *list)
 		return;
 	}
 
-	str = g_strdup_printf ("<b>%s</b>", group);
-
 	/* Translator: %s denotes the contact ID */
+	str = g_strdup_printf ("<b>%s</b>", group);
 	dialog = gtk_message_dialog_new (GTK_WINDOW (gossip_app_get_window ()),
 					 0,
 					 GTK_MESSAGE_QUESTION,
-					 GTK_BUTTONS_OK_CANCEL,
+					 GTK_BUTTONS_NONE,
 					 _("Please enter a new name for the group:\n%s"),
 					 str);
+	gtk_dialog_add_buttons (GTK_DIALOG (dialog), 
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				_("Rename"), GTK_RESPONSE_OK,
+				NULL);
 
 	g_free (str);
 
-	g_object_set (GTK_MESSAGE_DIALOG (dialog)->label,
-		      "use-markup", TRUE,
-		      NULL);
-
-	entry = gtk_entry_new ();
-	gtk_widget_show (entry);
-
-	gtk_entry_set_text (GTK_ENTRY (entry), group);
-	gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
-
-	g_signal_connect (entry,
-			  "activate",
-			  G_CALLBACK (contact_list_action_entry_activate_cb),
-			  dialog);
-
 	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox);
-
-	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 4);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    hbox, FALSE, TRUE, 4);
 
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
-		str = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
-	} else {
-		str = NULL;
-	}
+	entry = gtk_entry_new ();
+	gtk_entry_set_text (GTK_ENTRY (entry), group);
+	gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 4);
 
-	gtk_widget_destroy (dialog);
+	g_object_set (GTK_MESSAGE_DIALOG (dialog)->label, "use-markup", TRUE, NULL);
+	g_object_set_data (G_OBJECT (dialog), "entry", entry);
+	g_object_set_data_full (G_OBJECT (dialog), "group", group, g_free);
 
-	if (!str || !str[0]) {
-		g_free (group);
-		return;
-	}
+	g_signal_connect (entry, "activate",
+			  G_CALLBACK (contact_list_action_entry_activate_cb),
+			  dialog);
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (contact_list_action_rename_group_response_cb),
+			  list);
 
-	gossip_session_rename_group (priv->session, group, str);
-	g_free (group);
+	gtk_widget_show_all (dialog);
 }
 
 static void
@@ -2545,7 +2591,7 @@ contact_list_free_flash_timeout_data (FlashTimeoutData *timeout_data)
 
 	g_object_unref (timeout_data->contact);
 
-	g_free (timeout_data);
+	g_slice_free (FlashTimeoutData, timeout_data);
 }
 
 static void
@@ -2557,7 +2603,7 @@ contact_list_flash_free_data (FlashData *data)
 		g_source_remove (data->flash_timeout_id);
 	}
 
-	g_free (data);
+	g_slice_free (FlashData, data);
 }
 
 static gboolean
@@ -2685,12 +2731,12 @@ contact_list_event_added_cb (GossipEventManager *manager,
 		return;
 	}
 
-	timeout_data = g_new0 (FlashTimeoutData, 1);
+	timeout_data = g_slice_new0 (FlashTimeoutData);
 
 	timeout_data->list = list;
 	timeout_data->contact = g_object_ref (contact);
 
-	data = g_new0 (FlashData, 1);
+	data = g_slice_new0 (FlashData);
 
 	data->event_id = gossip_event_get_id (event);
 	data->event_type = type;
@@ -2858,7 +2904,7 @@ gossip_contact_list_get_selected (GossipContactList *list)
 	return contact;
 }
 
-char *
+gchar *
 gossip_contact_list_get_selected_group (GossipContactList *list)
 {
 	GossipContactListPriv *priv;
@@ -2907,7 +2953,6 @@ gossip_contact_list_set_show_offline (GossipContactList *list,
 				      gboolean           show_offline)
 {
 	GossipContactListPriv *priv;
-	GossipSession         *session;
 	const GList           *contacts;
 	const GList           *l;
 	gboolean               show_active;
@@ -2922,15 +2967,13 @@ gossip_contact_list_set_show_offline (GossipContactList *list,
 	/* Disable temporarily. */
 	priv->show_active = FALSE;
 
-	session = priv->session;
-	contacts = gossip_session_get_contacts (session);
+	contacts = gossip_session_get_contacts (priv->session);
 	for (l = contacts; l; l = l->next) {
 		GossipContact *contact;
 
 		contact = GOSSIP_CONTACT (l->data);
 
-		contact_list_contact_presence_updated_cb (session, contact,
-							  list);
+		contact_list_contact_update (list, contact);
 	}
 
 	/* Restore to original setting. */
