@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2006 Imendio AB
- * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <geertjan@gnome.org>
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,37 +16,47 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
+ *          Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-chatroom.h>
+#include <libgossip/gossip-session.h>
 #include <libgossip/gossip-debug.h>
-#include <libgossip/gossip-message.h>
+#include <libgossip/gossip-protocol.h>
+#include <libgossip/gossip-log.h>
 
 #include "gossip-app.h"
 #include "gossip-chat-window.h"
 #include "gossip-chat-view.h"
 #include "gossip-chat.h"
-#include "gossip-log.h"
 #include "gossip-preferences.h"
 #include "gossip-private-chat.h"
 #include "gossip-sound.h"
 #include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_PRIVATE_CHAT, GossipPrivateChatPriv))
-
-#define IS_ENTER(v) (v == GDK_Return || v == GDK_ISO_Enter || v == GDK_KP_Enter)
-
 #define DEBUG_DOMAIN "PrivateChat"
 
 #define COMPOSING_STOP_TIMEOUT 5
 
+#define IS_ENTER(v) (v == GDK_Return || v == GDK_ISO_Enter || v == GDK_KP_Enter)
+
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_PRIVATE_CHAT, GossipPrivateChatPriv))
+
 struct _GossipPrivateChatPriv {
 	GossipContact    *contact;
 	GossipContact    *own_contact;
@@ -78,11 +87,11 @@ static void           private_chat_composing_start              (GossipPrivateCh
 static void           private_chat_composing_stop               (GossipPrivateChat      *chat);
 static void           private_chat_composing_remove_timeout     (GossipPrivateChat      *chat);
 static gboolean       private_chat_composing_stop_timeout_cb    (GossipPrivateChat      *chat);
-static void           private_chat_contact_presence_updated     (gpointer                not_used,
-								 GossipContact          *contact,
+static void           private_chat_contact_presence_updated     (GossipContact          *contact,
+								 GParamSpec             *param,
 								 GossipPrivateChat      *chat);
-static void           private_chat_contact_updated              (gpointer                not_used,
-								 GossipContact          *contact,
+static void           private_chat_contact_updated              (GossipContact          *contact,
+								 GParamSpec             *param,
 								 GossipPrivateChat      *chat);
 static void           private_chat_contact_added                (gpointer                not_used,
 								 GossipContact          *contact,
@@ -165,20 +174,20 @@ gossip_private_chat_init (GossipPrivateChat *chat)
 
 	gossip_debug (DEBUG_DOMAIN, "Connecting");
 
-	g_signal_connect_object (gossip_app_get_session (),
-				 "protocol-connected",
-				 G_CALLBACK (private_chat_protocol_connected_cb),
-				 chat, 0);
+	g_signal_connect (gossip_app_get_session (),
+			  "protocol-connected",
+			  G_CALLBACK (private_chat_protocol_connected_cb),
+			  chat);
 
-	g_signal_connect_object (gossip_app_get_session (),
-				 "protocol-disconnected",
-				 G_CALLBACK (private_chat_protocol_disconnected_cb),
-				 chat, 0);
+	g_signal_connect (gossip_app_get_session (),
+			  "protocol-disconnected",
+			  G_CALLBACK (private_chat_protocol_disconnected_cb),
+			  chat);
 
-	g_signal_connect_object (gossip_app_get_session (),
-				 "composing",
-				 G_CALLBACK (private_chat_composing_cb),
-				 chat, 0);
+	g_signal_connect (gossip_app_get_session (),
+			  "composing",
+			  G_CALLBACK (private_chat_composing_cb),
+			  chat);
 }
 
 static void
@@ -192,13 +201,32 @@ private_chat_finalize (GObject *object)
 	chat = GOSSIP_PRIVATE_CHAT (object);
 	priv = GET_PRIV (chat);
 
-	g_signal_handlers_disconnect_by_func (priv->contact,
+	g_signal_handlers_disconnect_by_func (gossip_app_get_session (),
+					      private_chat_protocol_connected_cb,
+					      chat);
+	g_signal_handlers_disconnect_by_func (gossip_app_get_session (),
+					      private_chat_protocol_disconnected_cb,
+					      chat);
+	g_signal_handlers_disconnect_by_func (gossip_app_get_session (),
+					      private_chat_composing_cb,
+					      chat);
+
+	g_signal_handlers_disconnect_by_func (gossip_app_get_session (), 
+					      private_chat_contact_added,
+					      chat);
+
+	g_signal_handlers_disconnect_by_func (priv->own_contact,
 					      private_chat_own_avatar_notify_cb,
 					      chat);
 	g_signal_handlers_disconnect_by_func (priv->contact,
 					      private_chat_other_avatar_notify_cb,
 					      chat);
-		
+	g_signal_handlers_disconnect_by_func (priv->contact,
+					      private_chat_contact_updated,
+					      chat);
+	g_signal_handlers_disconnect_by_func (priv->contact,
+					      private_chat_contact_presence_updated,
+					      chat);
 
 	if (priv->contact) {
 		g_object_unref (priv->contact);
@@ -278,8 +306,6 @@ private_chat_create_ui (GossipPrivateChat *chat)
 			  "focus_in_event",
 			  G_CALLBACK (private_chat_text_view_focus_in_event_cb),
 			  chat);
-
-	gtk_widget_grab_focus (GOSSIP_CHAT (chat)->input_text_view);
 }
 
 static void
@@ -341,6 +367,7 @@ private_chat_send (GossipPrivateChat *chat,
 		   const gchar       *msg)
 {
 	GossipPrivateChatPriv *priv;
+	GossipLogManager      *log_manager;
 	GossipMessage         *message;
 
 	priv = GET_PRIV (chat);
@@ -370,7 +397,8 @@ private_chat_send (GossipPrivateChat *chat,
 	gossip_message_request_composing (message);
 	gossip_message_set_sender (message, priv->own_contact);
 
-	gossip_log_message_for_contact (message, FALSE);
+	log_manager = gossip_session_get_log_manager (gossip_app_get_session ());
+	gossip_log_message_for_contact (log_manager, message, FALSE);
 
 	gossip_chat_view_append_message_from_self (GOSSIP_CHAT (chat)->view,
 						   message,
@@ -443,18 +471,14 @@ private_chat_composing_stop_timeout_cb (GossipPrivateChat *chat)
 }
 
 static void
-private_chat_contact_presence_updated (gpointer           not_used,
-				       GossipContact     *contact,
+private_chat_contact_presence_updated (GossipContact     *contact,
+				       GParamSpec        *param,
 				       GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
 
 	priv = GET_PRIV (chat);
 
-	if (!gossip_contact_equal (contact, priv->contact)) {
-		return;
-	}
-
 	gossip_debug (DEBUG_DOMAIN, "Presence update for contact: %s",
 		      gossip_contact_get_id (contact));
 
@@ -493,18 +517,14 @@ private_chat_contact_presence_updated (gpointer           not_used,
 }
 
 static void
-private_chat_contact_updated (gpointer           not_used,
-			      GossipContact     *contact,
+private_chat_contact_updated (GossipContact     *contact,
+			      GParamSpec        *param,
 			      GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
 
 	priv = GET_PRIV (chat);
 
-	if (!gossip_contact_equal (contact, priv->contact)) {
-		return;
-	}
-
 	if (strcmp (priv->name, gossip_contact_get_name (contact)) != 0) {
 		g_free (priv->name);
 		priv->name = g_strdup (gossip_contact_get_name (contact));
@@ -528,14 +548,28 @@ private_chat_contact_added (gpointer           not_user,
 	g_signal_handlers_disconnect_by_func (priv->contact,
 					      private_chat_other_avatar_notify_cb,
 					      chat);
+	g_signal_handlers_disconnect_by_func (priv->contact,
+					      private_chat_contact_updated,
+					      chat);
+	g_signal_handlers_disconnect_by_func (priv->contact,
+					      private_chat_contact_presence_updated,
+					      chat);
 
 	g_object_unref (priv->contact);
 	priv->contact = g_object_ref (contact);
 
-	g_signal_connect (priv->contact,
+	g_signal_connect (priv->contact, 
 			  "notify::avatar",
 			  G_CALLBACK (private_chat_other_avatar_notify_cb),
 			  chat);
+	g_signal_connect (priv->contact, 
+			  "notify::name",
+			  G_CALLBACK (private_chat_contact_updated),
+			  chat);
+	g_signal_connect (priv->contact, 
+			  "notify::presences",
+			  G_CALLBACK (private_chat_contact_presence_updated),
+			  chat);
 }
 
 static void
@@ -897,8 +931,9 @@ GossipPrivateChat *
 gossip_private_chat_new (GossipContact *own_contact,
 			 GossipContact *contact)
 {
-	GossipPrivateChat     *chat;
 	GossipPrivateChatPriv *priv;
+	GossipPrivateChat     *chat;
+	GossipLogManager      *log_manager;
 	GossipChatView        *view;
 	GossipContact         *sender;
 	GossipMessage         *message;
@@ -917,15 +952,27 @@ gossip_private_chat_new (GossipContact *own_contact,
 
 	priv->name = g_strdup (gossip_contact_get_name (contact));
 
-	g_signal_connect (priv->own_contact,
+	g_signal_connect (priv->own_contact, 
 			  "notify::avatar",
 			  G_CALLBACK (private_chat_own_avatar_notify_cb),
 			  chat);
-
-	g_signal_connect (priv->contact,
+	g_signal_connect (priv->contact, 
 			  "notify::avatar",
 			  G_CALLBACK (private_chat_other_avatar_notify_cb),
 			  chat);
+	g_signal_connect (priv->contact, 
+			  "notify::name",
+			  G_CALLBACK (private_chat_contact_updated),
+			  chat);
+	g_signal_connect (priv->contact, 
+			  "notify::presences",
+			  G_CALLBACK (private_chat_contact_presence_updated),
+			  chat);
+
+	g_signal_connect (gossip_app_get_session (), 
+			  "contact_added",
+			  G_CALLBACK (private_chat_contact_added),
+			  chat);
 
 	private_chat_own_avatar_notify_cb (priv->own_contact, NULL, chat);
 	private_chat_other_avatar_notify_cb (priv->contact, NULL, chat);
@@ -936,7 +983,8 @@ gossip_private_chat_new (GossipContact *own_contact,
 	gossip_chat_view_scroll (view, FALSE);
 
 	/* Add messages from last conversation */
-	messages = gossip_log_get_last_for_contact (priv->contact);
+	log_manager = gossip_session_get_log_manager (gossip_app_get_session ());
+	messages = gossip_log_get_last_for_contact (log_manager, priv->contact);
 	num_messages  = g_list_length (messages);
 
 	for (l = messages, i = 0; l; l = l->next, i++) {
@@ -973,22 +1021,6 @@ gossip_private_chat_new (GossipContact *own_contact,
 					   private_chat_scroll_down_idle_func, 
 					   g_object_ref (chat));
 
-	/* Set up signals */
-	g_signal_connect_object (gossip_app_get_session (),
-				 "contact_presence_updated",
-				 G_CALLBACK (private_chat_contact_presence_updated),
-				 chat, 0);
-
-	g_signal_connect_object (gossip_app_get_session (),
-				 "contact_updated",
-				 G_CALLBACK (private_chat_contact_updated),
-				 chat, 0);
-
-	g_signal_connect_object (gossip_app_get_session (),
-				 "contact_added",
-				 G_CALLBACK (private_chat_contact_added),
-				 chat, 0);
-
 	priv->is_online = gossip_contact_is_online (priv->contact);
 
 	return chat;
@@ -1026,6 +1058,7 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 				    GossipMessage     *message)
 {
 	GossipPrivateChatPriv *priv;
+	GossipLogManager      *log_manager;
 	GossipContact         *sender;
 	GossipChatroomInvite  *invite;
 	const gchar           *resource;
@@ -1060,7 +1093,8 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 		}
 	}
 
-	gossip_log_message_for_contact (message, TRUE);
+	log_manager = gossip_session_get_log_manager (gossip_app_get_session ());
+	gossip_log_message_for_contact (log_manager, message, TRUE);
 
 	subject = gossip_message_get_subject (message);
 	if (subject) {
