@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2006 Imendio AB
+ * Copyright (C) 2003-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,16 +16,23 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
 
 #ifdef HAVE_GNOME
 #include <libgnome/gnome-sound.h>
 #include <libgnome/gnome-triggers.h>
 #endif
 
-#include <libgossip/gossip-session.h>
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-protocol.h>
+#include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-conf.h>
 
@@ -40,9 +47,16 @@
  */
 #define SOUND_WAIT_TIME 10000
 
-static void sound_contact_presence_updated_cb (GossipSession *session,
+static void sound_contact_presence_updated_cb (GossipContact *contact,
+					       GParamSpec    *param,
+					       gpointer       user_data);
+static void sound_contact_added_cb            (GossipSession *session,
 					       GossipContact *contact,
 					       gpointer       user_data);
+static void sound_contact_removed_cb          (GossipSession *session,
+					       GossipContact *contact,
+					       gpointer       user_data);
+static void sound_contact_remove              (GossipContact *contact);
 
 static GHashTable    *account_states = NULL;
 static GHashTable    *contact_states = NULL;
@@ -63,20 +77,26 @@ sound_protocol_connected_cb (GossipSession  *session,
 			     gpointer        user_data)
 {
 	guint        id;
-	const gchar *account_id;
+	const gchar *account_name;
+
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Protocol connected for account:'%s'",
+		      gossip_account_get_name (account));
 
 	if (g_hash_table_lookup (account_states, account)) {
 		return;
 	}
 
-	account_id = gossip_account_get_id (account);
-	gossip_debug (DEBUG_DOMAIN, "Account update, account:'%s' is now online",
-		      account_id);
+	account_name = gossip_account_get_name (account);
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Account update, account:'%s' is now online",
+		      account_name);
 
 	id = g_timeout_add (SOUND_WAIT_TIME,
 			    (GSourceFunc) sound_protocol_timeout_cb,
 			    account);
-	g_hash_table_insert (account_states, account, GUINT_TO_POINTER (id));
+	g_hash_table_insert (account_states, g_object_ref (account),
+			     GUINT_TO_POINTER (id));
 }
 
 static gboolean
@@ -96,12 +116,15 @@ sound_disconnected_contact_foreach (GossipContact  *contact,
 }
 
 static void
-sound_protocol_disconnected_cb (GossipSession  *session,
-				GossipAccount  *account,
-				GossipProtocol *protocol,
-				gint            reason,
-				gpointer        user_data)
+sound_protocol_disconnecting_cb (GossipSession  *session,
+				 GossipAccount  *account,
+				 GossipProtocol *protocol,
+				 gpointer        user_data)
 {
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Protocol disconnecting for account:'%s'",
+		      gossip_account_get_name (account));
+
 	g_hash_table_remove (account_states, account);
 
 	g_hash_table_foreach_remove (contact_states,
@@ -110,8 +133,8 @@ sound_protocol_disconnected_cb (GossipSession  *session,
 }
 
 static void
-sound_contact_presence_updated_cb (GossipSession *session,
-				   GossipContact *contact,
+sound_contact_presence_updated_cb (GossipContact *contact,
+				   GParamSpec    *param,
 				   gpointer       user_data)
 {
 	GossipPresence *presence;
@@ -123,7 +146,8 @@ sound_contact_presence_updated_cb (GossipSession *session,
 	presence = gossip_contact_get_active_presence (contact);
 	if (!presence) {
 		if (g_hash_table_lookup (contact_states, contact)) {
-			gossip_debug (DEBUG_DOMAIN, "Presence update, contact:'%s' is now offline",
+			gossip_debug (DEBUG_DOMAIN,
+				      "Presence update, contact:'%s' is now offline",
 				      gossip_contact_get_id (contact));
 			gossip_sound_play (GOSSIP_SOUND_OFFLINE);
 		}
@@ -140,7 +164,8 @@ sound_contact_presence_updated_cb (GossipSession *session,
 		 */
 		if (!g_hash_table_lookup (account_states, account) &&
 		    !g_hash_table_lookup (contact_states, contact)) {
-			gossip_debug (DEBUG_DOMAIN, "Presence update, contact:'%s' is now online",
+			gossip_debug (DEBUG_DOMAIN, 
+				      "Presence update, contact:'%s' is now online",
 				      gossip_contact_get_id (contact));
 			gossip_sound_play (GOSSIP_SOUND_ONLINE);
 		}
@@ -151,6 +176,36 @@ sound_contact_presence_updated_cb (GossipSession *session,
 	}
 }
 
+static void
+sound_contact_added_cb (GossipSession *session,
+			GossipContact *contact,
+			gpointer       user_data)
+{
+	g_signal_connect (contact, "notify::presences",
+			  G_CALLBACK (sound_contact_presence_updated_cb),
+			  NULL);
+	g_signal_connect (contact, "notify::type",
+			  G_CALLBACK (sound_contact_presence_updated_cb),
+			  NULL);
+}
+
+static void
+sound_contact_removed_cb (GossipSession *session,
+			  GossipContact *contact,
+			  gpointer       user_data)
+{
+	g_hash_table_remove (contact_states, contact);
+}
+
+static void
+sound_contact_remove (GossipContact *contact)
+{
+	g_signal_handlers_disconnect_by_func (contact,
+					      sound_contact_presence_updated_cb,
+					      NULL);
+	g_object_unref (contact);
+}
+
 void
 gossip_sound_play (GossipSound sound)
 {
@@ -249,20 +304,20 @@ gossip_sound_init (GossipSession *session)
 
 	contact_states = g_hash_table_new_full (gossip_contact_hash,
 						gossip_contact_equal,
-						(GDestroyNotify) g_object_unref,
+						(GDestroyNotify) sound_contact_remove,
 						(GDestroyNotify) g_object_unref);
 
-	g_signal_connect (session,
-			  "protocol-connected",
+	g_signal_connect (session, "protocol-connected",
 			  G_CALLBACK (sound_protocol_connected_cb),
 			  NULL);
-	g_signal_connect (session,
-			  "protocol-disconnected",
-			  G_CALLBACK (sound_protocol_disconnected_cb),
+	g_signal_connect (session, "protocol-disconnecting",
+			  G_CALLBACK (sound_protocol_disconnecting_cb),
 			  NULL);
-	g_signal_connect (session,
-			  "contact-presence-updated",
-			  G_CALLBACK (sound_contact_presence_updated_cb),
+	g_signal_connect (session, "contact-added",
+			  G_CALLBACK (sound_contact_added_cb),
+			  NULL);
+	g_signal_connect (session, "contact-removed",
+			  G_CALLBACK (sound_contact_removed_cb),
 			  NULL);
 }
 
